-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, IEE = ...

-- Create the options namespace.
local options = { }

-- Expose the options to the IgnoreEnemyEmotes Addon Space.
IEE.options = options

-------------------------------------------------------------------------------------
-- | Variable Definitions |----------------------------------------------------------
-------------------------------------------------------------------------------------

-- Active options, these represent the live, non-cache states of the addon.
-- These should be read from the IgnoreEnemyEmotesDB on loading, and flushed to it on unloading.
-- These should be read from the cache on confirming changes.
options.savedVars = { }
options.savedVars["onlyInPvP"] = false

-- Cached options.
-- These should be flushed to the savedVars on confirming changes.
-- These should be cleared when leaving the options menu.
local cache = { }
cache["onlyInPvP"] = nil

-- Default options. These cannot be changed after being set.
-- These must be explicitly defined for every savedVar.
local defaults = { }
defaults["onlyInPvP"] = false

-------------------------------------------------------------------------------------
-- | Event Hooks and Handlers |------------------------------------------------------
-------------------------------------------------------------------------------------

-- Initializes the options menu.
-- To be called after ADDON_LOADED
-- Exposed to Addon to let IgnoreEnemyEmotes.lua call it.
function IEE.initializeOptionsMenu()
  
  
  -- Prepare the options panel for adding to the Interface->Addons menu.
  local optionsPanel    = IgnoreEnemyEmotesOptionsPanel
  optionsPanel.name     = "IgnoreEnemyEmotes"
  
  -- On okay, flush all changes in cache to savedVars.
  optionsPanel.okay     = function()
    for k,v in pairs(cache) do
      options.savedVars[k] = v
      cache[k] = nil
    end
  end
  
  -- On default, reset all in live to the default.
  optionsPanel.default  = function()
    for k,v in pairs(defaults) do
      options.savedVars[k] = defaults[k]
    end
	end
  
  -- On cancel, reset the cache.
  optionsPanel.cancel   = function()
    for k,_ in pairs(cache) do
      cache[k] = nil
    end
  end
  
  getglobal(optionsPanel.CheckOutdoorToggle:GetName() .. "Text"):SetText("Only while in Battlegrounds and Arenas")
  optionsPanel.CheckOutdoorToggle:SetScript("OnShow",    function() optionsPanel.CheckOutdoorToggle:SetChecked(options.savedVars["onlyInPvP"]) end)
  optionsPanel.CheckOutdoorToggle:SetScript("OnHide",    function() cache["onlyInPvP"] = nil end)
  optionsPanel.CheckOutdoorToggle:SetScript("PostClick", function() cache["onlyInPvP"] = optionsPanel.CheckOutdoorToggle:GetChecked() end)
  optionsPanel.CheckOutdoorToggle:SetScript("OnClick",   function() end) -- An On_Click handler must be defined or Blizzard Code throws an error.
  
  -- Add the options panel to the Interface->Addons menu.
	InterfaceOptions_AddCategory(optionsPanel)
  
  -- Read all savedVars from either IgnoreEnemyEmotesDB or their default value.
  for k, _ in pairs(defaults) do
    options.savedVars[k] = IgnoreEnemyEmotesDB[k] or defaults[k]
  end
end

-- Flushes all options.savedVars to IgnoreEnemyEmotesDB.
-- To be called on a reload or logout.
-- Exposed to Addon to let IgnoreEnemyEmotes.lua call it.
function IEE.unloadOptions()
  -- Write all savedVars to IgnoreEnemyEmotesDB
  for k, _ in pairs(options.savedVars) do
    IgnoreEnemyEmotesDB[k] = options.savedVars[k]
  end
  
end

-------------------------------------------------------------------------------------
-- | Testing Functionality |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Test function.
-- Could expand to only show failed cases.
options.fullTest = function()
  local faction = UnitFactionGroup("player")
  
  print("IEE Testing: Full Test for " .. faction)
   
  local pvpOptions  = { true, false }
  local optOptions  = { true, false }
  local raceOptions = { "Human", "Orc", "Pandaren" }
  local langOptions = { "Common", "Orcish" }
  
  local testID = 0
  local result = nil
  local expectedResults = {
    [ 1] = { ["Horde"] =  true, ["Alliance"] = false, },
    [ 2] = { ["Horde"] = false, ["Alliance"] = false, },
    [ 3] = { ["Horde"] =  true, ["Alliance"] = false, },
    [ 4] = { ["Horde"] =  true, ["Alliance"] = false, },
    [ 5] = { ["Horde"] =  true, ["Alliance"] = false, },
    [ 6] = { ["Horde"] = false, ["Alliance"] = false, },
    [ 7] = { ["Horde"] =  true, ["Alliance"] = false, },
    [ 8] = { ["Horde"] =  true, ["Alliance"] = false, },
    [ 9] = { ["Horde"] = false, ["Alliance"] =  true, },
    [10] = { ["Horde"] = false, ["Alliance"] = false, },
    [11] = { ["Horde"] = false, ["Alliance"] =  true, },
    [12] = { ["Horde"] = false, ["Alliance"] =  true, },
    [13] = { ["Horde"] = false, ["Alliance"] =  true, },
    [14] = { ["Horde"] = false, ["Alliance"] = false, },
    [15] = { ["Horde"] = false, ["Alliance"] =  true, },
    [16] = { ["Horde"] = false, ["Alliance"] =  true, },
  }
  local fails = 0
  
  local onlyInPvPVar = options.savedVars["onlyInPvP"] -- store the original saved var as we'll adjust it during testing and have to set it back.
  
  -- perform tests.
  for _, l in ipairs(langOptions) do
    for _, r in ipairs(raceOptions) do
      for _, o in ipairs(optOptions) do
        for _, p in ipairs(pvpOptions) do
          if  not ((l == "Common" and r == "Orc") -- Exclude testcase because of character limit.
              or   (l == "Orcish" and r == "Human")) then
            -- Perform Test
            testID = testID + 1
            options.savedVars["onlyInPvP"] = o
            result = IEE.ignoreEnemyEmotesFilter(r, l, p)
            
            -- If failed, print conditions
            if result ~= expectedResults[testID][faction] then
              fails = fails + 1
              print("Failed Test Case: " .. testID)
              print(p and "In PvP" or "Outdoors")
              print("Filter: " .. (o and "Only in PvP" or "Everywhere"))
              print("Race: " .. r)
              print("Language: " .. l)

              print("Message Filtered: " .. ((result) and "Yes" or "No"))
            end
          end
        end
      end
    end
  end
  
  -- Report results
  print("Testing ended with " .. tostring(fails) .. " fails")
  
  options.savedVars["onlyInPvP"] = onlyInPvPVar
end

-- eof